# -*- coding: utf-8 -*-

'''
Created 19-Dec-2024
Updated 24-Dec-2024
'''

'''
This Python file is an example that shows how to use the ChildElemIter class to enumerate a complex element. 
A complex element could be grouped hole, complex chain, complex shape or nested cell.
In this example we consider only grouped holes and normal cells.
Supply the element ID of a grouped hole and a nested cell in the __main__ function at the end of this file.
'''


from MSPyBentley import *
from MSPyDgnPlatform import *
from MSPyMstnPlatform import *

def getElementDescription (element, stringLength: int = 32)->str:
    '''
    Get the human-readable description of an element from an elementRef or ElementHandle.
    '''
    description = WString()
    if isinstance (element, ElementHandle):
        handler = element.GetHandler()
        if handler is None:
            MessageCenter.ShowErrorMessage(f"getElementDescription() ElementHandle is None!", None, False)
        else:
            handler.GetDescription (element, description, stringLength)
        
    elif isinstance (element, PersistentElementRef):
        eh = ElementHandle (element, dgnModel)
        handler = eh.GetHandler()       
        handler.GetDescription (eh, description, stringLength)
        
    return str(description)

# ExposeChildrenReason instance required by the ChildElemIter() constructor.
# See: https://bentleysystems.service-now.com/community?id=community_question&sys_id=3950e8461b6612147171ff7e034bcb58
ExposeChildrenCount = ExposeChildrenReason(100)
ExposeChildrenQuery = ExposeChildrenReason(200)
ExposeChildrenEdit  = ExposeChildrenReason(300)

def AnalyseNestedCell (eh, nestDepth = 0):
    # Step through the components of a grouped hole or normal cell.
    # If the cell contains nested components, this function calls
    # itself recursively
    n = 0 
    component = ChildElemIter(eh, ExposeChildrenCount)
    while component.IsValid():
        n += 1
        description = getElementDescription(component)
        MessageCenter.ShowDebugMessage(f"[{nestDepth}.{n}] Found component {description}", f"[{nestDepth}.{n}] Found component {description}", False)
        # If this cell contains one or more nested cells, then analyse them recursively
        if  isinstance (component.GetHandler(), ComplexElementHandlers):
            AnalyseNestedCell (component, nestDepth + 1)
        else:
            # Analyse primitive element
            pass
            
        component = component.ToNext()
 
# A Python tuple of DGN element handlers    
ComplexElementHandlers = (GroupedHoleHandler, NormalCellHeaderHandler, ) 
   
#main
if __name__ == "__main__":

    # Modify these IDs to match those of a grouped hole and a nested cell in your DGN model
    GROUPED_HOLE_ELEMENTID = 1184
    NESTED_CELL_ELEMENTID = 1629
        
    eh = ElementHandle (GROUPED_HOLE_ELEMENTID, ISessionMgr.ActiveDgnModelRef.GetDgnModel())
    description = getElementDescription(eh)
    MessageCenter.ShowDebugMessage(f"Element ID {GROUPED_HOLE_ELEMENTID} is {description}", f"Element ID {GROUPED_HOLE_ELEMENTID} is {description}", False)
    if eh.IsValid():
        MessageCenter.ShowDebugMessage(f"Element ID {GROUPED_HOLE_ELEMENTID} is valid",
             f"Element ID {GROUPED_HOLE_ELEMENTID} is a {description}", False)
        if  isinstance (eh.GetHandler(), ComplexElementHandlers):
            MessageCenter.ShowDebugMessage(f"Element is grouped hole", None, False)
            AnalyseNestedCell (eh)
                
        else:
            MessageCenter.ShowErrorMessage(f"Element ID {GROUPED_HOLE_ELEMENTID} is not a complex element", None, False)
    else:
        MessageCenter.ShowErrorMessage(f"Invalid element ID {GROUPED_HOLE_ELEMENTID}", None, False)
        
    eh = ElementHandle (NESTED_CELL_ELEMENTID, ISessionMgr.ActiveDgnModelRef.GetDgnModel())
    if eh.IsValid():
        description = getElementDescription(eh)
        MessageCenter.ShowDebugMessage(f"Element ID {NESTED_CELL_ELEMENTID} is {description}", f"Element ID {NESTED_CELL_ELEMENTID} is {description}", False)
        if  isinstance (eh.GetHandler(), ComplexElementHandlers):
            MessageCenter.ShowDebugMessage(f"Element is complex", None, False)
            AnalyseNestedCell (eh)          
    else:
        MessageCenter.ShowErrorMessage(f"Invalid element ID {NESTED_CELL_ELEMENTID}", None, False)
